/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* real time tracking routines
*
**************************************************************************/
#include <curses.h>
#include "defs.h"
#include "lib.p"
#include "iptrk.p"

/**************************************************************************
*
**************************************************************************/
PRIVATE void display_stream_grams_sub(
  U_32 src, U_32 dst, U_16 vol, U_8 prot, U_16 port_s, U_16 port_d,
  int pack_dir)
{
  char           str[STR_LEN + 1], prot_str[5];
  struct in_addr in;

  switch(prot)
  {
/*the 3 common ones*/
    case IPPROTO_TCP:
      strcpy(prot_str, " TCP");
      break;

    case IPPROTO_UDP:
      strcpy(prot_str, " UDP");
      break;

    case IPPROTO_ICMP:
      strcpy(prot_str, "ICMP");
      break;

/*lesser protocols*/
    case IPPROTO_IPIP:
      strcpy(prot_str, "IPIP");
      break;

    case IPPROTO_GRE:
      strcpy(prot_str, " GRE");
      break;

    case IPPROTO_IPV6:
      strcpy(prot_str, "IPV6");
      break;

    case IPPROTO_EGP:
      strcpy(prot_str, " EGP");
      break;

    case IPPROTO_IGMP:
      strcpy(prot_str, "IGMP");
      break;

    case IPPROTO_PUP:
      strcpy(prot_str, " PUP");
      break;

    case IPPROTO_IDP:
      strcpy(prot_str, " IDP");
      break;

    case IPPROTO_RSVP:
      strcpy(prot_str, "RSVP");
      break;

    case IPPROTO_PIM:
      strcpy(prot_str, " PIM");
      break;

    case IPPROTO_ESP:
      strcpy(prot_str, " ESP");
      break;

    case IPPROTO_AH:
      strcpy(prot_str, "  AH");
      break;

    case IPPROTO_COMP:
      strcpy(prot_str, "COMP");
      break;

    case IPPROTO_IP:
      strcpy(prot_str, "  IP");
      break;

    case IPPROTO_RAW:
      strcpy(prot_str, " RAW");
      break;

    default:
      strcpy(prot_str, "????");
      break;
  };

  switch(pack_dir)
  {
    case DIR_I:
      wattrset(win_gram, COLOR_PAIR(COLOUR_GREEN));
      wattron(win_gram, A_BOLD);
      break;

    case DIR_O:
      wattrset(win_gram, COLOR_PAIR(COLOUR_GREEN));
      wattroff(win_gram, A_BOLD);
      break;

    case DIR_u:
      wattrset(win_gram, COLOR_PAIR(COLOUR_CYAN));
      wattroff(win_gram, A_BOLD);
      break;

    case DIR_m:
      wattrset(win_gram, COLOR_PAIR(COLOUR_WHITE));
      wattroff(win_gram, A_BOLD);
      break;

    case -1:
      wattrset(win_gram, COLOR_PAIR(COLOUR_BLUE));
      break;
  }

  in.s_addr = htonl(src);
  sprintf(str, "\n%-15s ", inet_ntoa(in));
  waddstr(win_gram, str);
  in.s_addr = htonl(dst);
  sprintf(str, "%-15s ", inet_ntoa(in));
  waddstr(win_gram, str);
  sprintf(str, "%5d %4s %5d %5d", vol, prot_str, port_s, port_d);
  waddstr(win_gram, str);
  wrefresh(win_gram);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void display_stream_grams(
  U_32 src, U_32 dst, U_16 vol, U_8 prot, U_16 port_s, U_16 port_d,
  int pack_dir)
{
  static int display_cyc;

  if(filter_src.addr != 0xffffffff)
    if(filter_src.addr != (src & filter_src.mask))
      return;

  if(filter_dst.addr != 0xffffffff)
    if(filter_dst.addr != (dst & filter_dst.mask))
      return;

  if(stream_factor == SF_MAX)
  {
    display_stream_grams_sub(src, dst, vol, prot, port_s, port_d, pack_dir);
    return;
  }

  if(display_cyc > (rate_gram_avg >> stream_factor))
  {
    display_cyc = 0;
    display_stream_grams_sub(src, dst, vol, prot, port_s, port_d, pack_dir);
  }
  else
    display_cyc++;
}
